# $Id$

require File.join(File.dirname(__FILE__), %w[spec_helper])

$: << File.join(File.split(File.dirname(__FILE__))[0..-2],'lib')
br_dir = File.expand_path File.join(File.split(File.expand_path(File.dirname(__FILE__)))[0..-2])

require 'backrack_client'
require 'json'


describe "Backrack client" do
  before(:all) do
   puts "brdir: #{br_dir}"
   puts "mydir: #{File.split(File.expand_path(File.dirname(__FILE__)))[0..-2]}"
   Dir.chdir("#{br_dir}/examples")
   @pid = Process.fork do     
     exec "#{br_dir}/bin/backrack -p 5000"
   end
   sleep 4 # let it get up
   @client = BackrackClient.new 'http://localhost:5000'   
   Foo = @client.get_class 'Foo'
  end
  
 before(:each) do
   puts "before eeee"
   @foo = Foo.new
 end
 
 it "should import methods" do
   @foo.methods.should include('bar')
 end
 
 it "should import class methods" do
   Foo.methods.should include('bar')
 end
 
 it "should properly work with many classes" do
   Woo = @client.get_class 'Woo'
   woo = Woo.new
   woo.a.should eql(222)
 end
 
 it "should execute object method" do
   @foo.bar.should eql("object method")
 end
 
 it "should execute class method" do
   Foo.bar.should eql("class method")
 end
 
 it "should use original initializer" do
   Boo = @client.get_class 'Boo'
   foo = Boo.new(3,4)
   foo.a.to_s.should eql('3')
   foo.b.to_s.should eql('4')
 end
 
 it "should raise RPCError exception on error" do
   raiser = false
   begin
    @foo.raiser
   rescue RPCError
     raiser = true
   end
   raiser.should eql(true)
 end
 
 it "should return proper values" do
   x,y = rand(100), rand(100)
   @foo.foobar(x,y).should == JSON.load({x => y}.to_json)
 end
 
 it "should properly store value in the object" do
   id = rand(100)
   @foo.set_id(id)
   @foo.id eql(id)
 end
 
 it "should remember which object is which" do
   @f1 = Foo.new
   id1 = @f1.id
   @f2 = Foo.new
   id2 = @f2.id  
   @f1.id.should eql(id1)
   @f2.id.should eql(id2)
 end
 
 it "should work with many arguments" do
   @foo.test_args(1,2,'q').should == JSON.load([1,2,'q'].to_json)
 end
 
 it "should invoke methods from parent class" do
   Child = @client.get_class 'Child'
   @child = Child.new
   @child.should respond_to(:foo)
 end
 
 it "should work with classes that have names with upcase letter in the middle" do
   FancyName = @client.get_class "FancyName"
   FancyName.new.foo.should == "foo"
 end
 
 it "should work with class methods with many parameters" do
   Foo.boo(2,3).should == 3
 end
 
 it "should work with unicode" do
   f = Foo.new
   str = "ąściółkęźż"
   f.print(str).should == str
 end
 
 after(:all) do
   Process.kill("TERM",@pid)   
 end
end

# EOF
