require 'remarkably/engines/html'

class Remarkably::Base::Engine
  def p(*args,&block)
    missing_method( :p, self, *args, &block)
  end
end

module Remarkably::Common
  def p(*args, &block)
    @remarkably_engine ||= Remarkably::Config.default_engine.new
    @remarkably_engine.send( :p, *args, &block )
  end
end

module Remarkably
  module Engines
    class XML < Base::Engine
      def _indentation(add=0)
        @indent||=0
        " " * (@indent+add)
      end
      
      def method! sym, args, hash, &block
        sym = sym.to_s.downcase
        
        if sym == "_"
          @output << (_indentation + args.join + "\n")
          return self
        end

        tag_attributes =
          hash.inject([]){|s,(k,v)| s << %{#{k.to_s.downcase}="#{v}"} }

        @output << _indentation + ["<#{sym}", tag_attributes].flatten.grep(/\S/).join(' ')

        if block_given? or not args.empty?
          @output << ">\n"
          
          @indent+=2
          block.call if block_given?
          @indent-=2
          
          @output << _indentation(2) + "#{args.join}\n" unless args.empty?
          @output << "#{_indentation}</#{sym}>\n"
        else
          @output << "/>\n"
        end

        self
      end
    end
  end
end


class Markup
  include Remarkably::Common
  
  def index
    html do
      head do
        title "example markup using patched remarkably"
      end
      body do
        p "this is what a patch is for - using p and not P"
        div(:id => "fufu") do
          p do
            _"here we'll use partial: "
            say_something_nice
          end
          _"and mayby some table"
          write_table
        end
      end
    end
  end
  
  def say_something_nice
    p "hello my little motherfuckers"
  end
  
  def write_table
    table{
      thead{
        tr{
          th "jeden"
          th "dwa"
        }
      }
      tbody{
        tr{
          td "1"
          td "2"
        }
      }
    }
  end
end
