require 'open4'
class Runner
  include Open4
  
  attr_accessor :on_close
  attr_reader :status
  attr_reader :pid
  
  @@instances = {}
  
  def self.register(iid,instance)
    @@instances[iid.to_s]=instance
  end
  
  def self.unregister(iid)
    @@instances[iid.to_s]=nil
  end
  
  def self.instance(iid)
    @@instances[iid.to_s] || self.new
  end
  
  def self.list
    
  end
  
  def initialize
    @writing_queue = Queue.new
    @std_queue = Queue.new
    @err_queue = Queue.new
    @status = :idle
    
    self.register(self.object_id, self)
  end
  
  def finished?
    @status == :finished
  end
  
  def write(line)
    Backrack.logger.debug("w > #{line}")
    @writing_queue.push("#{line}\n") if @status == :running
    self
  end
  
  alias << write
  
  def read
    res = ""
    while @std_queue.length > 0
      res += @std_queue.pop
    end   
    Backrack.logger.debug("r > #{line}")
        
    res
  end
  
  def read_err
    res = ""
    while @err_queue.length > 0
      res += @err_queue.pop
    end      
   Backrack.logger.debug "re > #{line}"
    
    res
  end
  
  def run(path, *params)
    return if @status == :running
    path = path+" "+params.flatten.join(" ")
    
    Thread.new do
      status = Open4::popen4(path) do |pid, stdin, stdout, stderr|
        @pid = pid
        @status = :running
        self.class.register(pid, self)
        
        @writer = Thread.new() do
          while (line = @writing_queue.pop)
            stdin.write line
            stdin.flush
          end
        end  
      
        @err_reader = Thread.new do
          while (line = stderr.gets)
            @err_queue.push line
          end
        end
      
        while (line = stdout.gets)
          @std_queue.push line
        end
      end
      @writing_queue.push(nil)
      
      # cleanup
      
      @exitstatus = status.exitstatus
      @on_close.call rescue nil
      @status = :finished
      
      @pid = nil
    end
    self
  end
  
  def wait(seconds=30)
    catch(:exited) {
      (seconds*2).times {
        throw(:exited, true) if @status==:finished
        sleep 0.5
      }
      false
    }
  end
  
  def kill
    Process.kill("TERM", @pid)
  end
  
  def result
    read
  end
end