$LOAD_PATH << File.expand_path("../lib", File.dirname(__FILE__))  
$LOAD_PATH << File.expand_path("../lib/backrack", File.dirname(__FILE__))  

require 'rubygems'
require 'logger'
require 'rack'
require 'parse_tree'

unless defined? Backrack
  module Backrack
    # :stopdoc:
    VERSION = '0.9.11'
    LIBPATH = ::File.expand_path(::File.dirname(__FILE__)) + ::File::SEPARATOR
    PATH = ::File.dirname(LIBPATH) + ::File::SEPARATOR
    # :startdoc:
    
    # lookout this is instance variable of module metaclass!!!!
    @config ||= {}
    
    class << self
      attr_accessor :logger
      attr_accessor :root
      attr_accessor :config
      
      def initialize!
        @config = {
          :root => File.expand_path("../", File.dirname(__FILE__)),
          :app_root => Dir.pwd,
          :path => Dir.pwd,
          :view => Dir.pwd,
          :env => "development",
          :port => 5000,
          :host => "0.0.0.0",
          :access_log => [],
          :session_expiration_time => 600000,
          :provide => [],
          :log_file => STDERR,
        }.merge(@config)  
        
        if @config[:path] != @config[:app_root] && @config[:path][0..0]!= "/"
          @config[:path] = File.join(@config[:app_root].to_s, @config[:path])
        end
        if @config[:view] != @config[:app_root] && @config[:view][0..0]!= "/"
          @config[:view] = File.join(@config[:app_root].to_s, @config[:view])
        end
        
        @logger = Logger.new(@config[:log_file])
        @logger.level = $DEBUG ? Logger::DEBUG : Logger::INFO
        @root = File.expand_path("../", File.dirname(__FILE__))  

        Dir.glob(File.join(File.dirname(__FILE__), "backrack/class_extensions", "*.rb")).each { |rb|
          require(rb)
        }

        ["uuid", "mimeparse", "object_manager", "dispatcher"].each do |lib|
          require "backrack/#{lib}"
        end

        @logger.info "Started Backrack"
        @logger.debug("with configuraction: #{@config.pretty_inspect}") if $DEBUG
        
        (@config[:load_path] || []).each do |path|
          $: << path
        end
        @logger.info "Loading initializers"
        (@config[:load] || []).each do |lib|
          require lib
        end
        @logger.info "finished initialization!"
      end
      
      def default_class
        @default_class ||= @config[:default_class]
      end
    
      def default_class=(dc)
        @default_class = dc
      end

      def load_rpc
        load_serializers(["xml_rpc", "json_rpc"])
      end

      def load_rest
        load_serializers(["text_proxy", "html", "xml", "json"])
      end
      
      def load_serializers(list = nil, reload=false)
        if reload
          list = @current_serializers
        else
          unless list
            backrack_serializers = File.join(@config[:root],"serializers","*.rb")
            app_serializers = File.join(@config[:app_root],"serializers","*.rb")
            list = Dir.glob(backrack_serializers) + Dir.glob(app_serializers)
          else
            list.map!{|name|
              serializer = File.join("serializers", name+".rb")
              if File.exist?(File.join(@config[:app_root], serializer))
                File.join(@config[:app_root], serializer)
              elsif File.exist?(File.join(@config[:root], serializer))
                File.join(@config[:root], serializer)
              else
                serializer
              end
            }
          end
          @current_serializers = list
        end
        
        @logger.debug "looking for serializers in #{app_serializers} and #{backrack_serializers}"

        list.each do |file|
          @logger.debug "loading serializer #{file}" 
          load file
        end
      end
      
      def authenticate(user,password)
        user==self.config[:http_user] && password == self.config[:http_password]
      end
    
      def discover(reload = false)
        Backrack::ObjectManager.discover!(reload)
      end
    end
    
    # Returns the version string for the library.
    #
    def self.version
      VERSION
    end

    # Returns the library path for the module. If any arguments are given,
    # they will be joined to the end of the libray path using
    # <tt>File.join</tt>.
    #
    def self.libpath( *args )
      args.empty? ? LIBPATH : ::File.join(LIBPATH, *args)
    end

    # Returns the lpath for the module. If any arguments are given,
    # they will be joined to the end of the path using
    # <tt>File.join</tt>.
    #
    def self.path( *args )
      args.empty? ? PATH : ::File.join(PATH, *args)
    end

    # Utility method used to rquire all files ending in .rb that lie in the
    # directory 
    def self.require_all_libs( fname, dir = nil )
      search_me = ::File.expand_path(
        ::File.join(fname, '*.rb'))

      Dir.glob(search_me).sort.each {|rb| require rb}
    end

    # Utility method used to rquire all files ending in .rb that lie in the
    # directory below this file that has the same name as the filename passed
    # in. Optionally, a specific _directory_ name can be passed in such that
    # the _filename_ does not have to be equivalent to the directory.
    #
    def self.require_all_libs_relative_to( fname, dir = nil )
      dir ||= ::File.basename(fname, '.*')
      search_me = ::File.expand_path(
        ::File.join(::File.dirname(fname), dir, '**', '*.rb'))
      
      Dir.glob(search_me).sort.each {|rb| require rb}
    end
  end  # module Backrack
end  # unless defined?
