module BoundAndUnbound
  def to_sexp
    method_name = self.inspect 
    if (method_name =~        /<Method: ([\w:]*)\(?([\w:]*)\)?(\#|\.)(.*)>/) || 
       (method_name =~ /<UnboundMethod: ([\w:]*)\(?([\w:]*)\)?(\#|\.)(.*)>/) then
      klass = ($2.empty? ? $1 : $2).to_object
      method  = $4.intern
      klass = klass.metaclass if $3 == '.'              
      raise "Couldn't determine class from #{method_name}" if klass.nil?
      ParseTree.new(false).parse_tree_for_method(klass, method)
    else
      raise "Can't parse signature: >#{method_name}< "
    end
  end
  
  def params(sexp = nil)
    sexp ||= self.to_sexp
    pr = sexp[2][1][1][1..-1] rescue []
    pr.select{|prr| prr.class == Symbol}
  end
    
  def type_annotations(sexp = nil)
    sexp ||= self.to_sexp
    ann = sexp[2][1][2] rescue []
    # unless first thing in method declaration is hash return nil
    return(nil) unless ann[0] == :hash
    ann.shift
    
    # we check if hash is in format {:symbol => Const}
    # or {var_name => Const}
    rot = false
    return(nil) unless ann.all?{|a| 
      rot = !rot; 
      rot ? ((a[0] == :lit) || (a[0] == :lvar)) : (a[0] == :const)  
    }

    # we change internal hash to ruby hash
    ann_hash = {}
    0.step(ann.length-1, 2) { |i| 
      ann_hash[ann[i][1]] = ann[i+1][1]
    }
    
    return(ann_hash)
  end
end

class Method
  include BoundAndUnbound
end

class UnboundMethod
  include BoundAndUnbound
end