require 'object_manager'

module Backrack
  module Helpers
    module OnePerSession
      def instance(session_key)
        @session_instances[session_key] ||= Backrack::ObjectManager.register(new, session_key)
      end

      def self.extended(c)
        c.send(:instance_variable_set, "@session_instances", {})
        c.send(:define_method, :session) do
          Session.instance(@_session_key)
        end
        c.metaclass.send(:private, :new) if c.respond_to? :new
      end
    end

    module ManyPerSession
      def create(session, *params)
        o = new(session, *params)
        Backrack::ObjectManager.register(o, session.session_key)
        id = o.respond_to?(:id) ? o.id : o.object_id
        @session_instances[session.session_key]||={}
        @session_instances[session.session_key][id]=o
        return(o)
      end

      def find(session, id)
        @session_instances[session.session_key] && @session_instances[session.session_key][id]
      end

      def find_object(session_key, id)
        find_or_create(Session[session_key], id, id)
      end

      def find_or_create(session, id, *params)
        o = find(session, id) || create(session, *params)
        return(o)
      end

      def self.extended(c)
        c.send(:instance_variable_set, "@session_instances", {})
        c.send(:define_method, :session) do
          Session.instance(@_session_key)
        end
        c.send(:extend, Backrack::Helpers)
      end
    end


    module ClassMethods
      def has(name, options={}, &block)
        cname = (options[:class_name] || name).to_s
        cname.camelize! unless ("A".."Z") === cname[0].chr
        conditional = options[:if]
        define_method(name) do
          klass = cname.to_object
          if conditional
            if (conditional.respond_to?(:call) ? conditional.call : self.send(conditional))
              klass.instance(@_session_key)
            end
          else
            klass.instance(@_session_key)
          end
        end
      end
    end

    module InstanceMethods

    end

    def self.included(receiver)
      receiver.extend         ClassMethods
      receiver.send :include, InstanceMethods
    end
  end
end