unless defined?(Backrack::ObjectManager)
  class Backrack::ObjectManager
    # Class methods
    class << self
      def init
        @cache ||= {}
        @instances ||= {}
        @classes_hash ||= {}
        @finders ||= {}
        @@mtime ||= {}

        Thread.new {
          loop {
            sleep 1000
            Backrack::ObjectManager.sweep
          }
        }
      end
    
      def discover!(use_load=false)
        app_path = Backrack.config[:path]
        Backrack.logger.debug "looking for stuff in #{app_path}"
    
        old_classes = []
        ObjectSpace.each_object(Module) do |klass|
          old_classes << klass
        end
        @new_classes = []
    
        if File.directory? app_path
          Dir.glob(app_path+"/*.rb") { |filename|
            Backrack.logger.debug "loading #{filename}"
            if @@mtime[filename] != File.mtime(filename)
              use_load ? Kernel.load(filename) : require(filename)
              @@mtime[filename] = File.mtime(filename)
            end   
          }
        else
          use_load ? Kernel.load(app_path) : require(app_path)
        end
    
        new_classes = []
        ObjectSpace.each_object(Module) do |klass|
          new_classes << klass
        end
    
        new_classes = new_classes - old_classes
        Backrack.logger.debug("New classes found: #{new_classes}") unless new_classes.empty?
    
        new_classes.each do |klass|
          register_class(klass) 
        end
      end
  
      def classes
        @classes_hash.keys
      end
  
      def classes_hash
        @classes_hash
      end  
    
      def get_class(name)
        @classes_hash[name]
      end
    
      def supported?(class_name)
        !!@classes_hash[class_name]
      end
    
      def register_class(klass, &block)
        block ||= if defined?(Backrack::Helpers::OnePerSession) && klass.kind_of?(Backrack::Helpers::OnePerSession)
          lambda{|id, key| klass.instance(key)}
        elsif klass.respond_to?(:instance)
          lambda{|id, key| klass.instance()}
        elsif klass.respond_to?(:find_object)
          lambda{|id, key| klass.find_object(key, id)}
        else
          lambda {|object_id, key|
            if object_id && object_id.to_i != 0
              begin
                o = ObjectSpace._id2ref(object_id.to_i) 
                unless o.class == klass
                  raise(RuntimeError, "expecting #{klass.name} got object of #{o.class}") 
                end
                return(o) # NEVER remove this !!!!
              rescue RangeError
                raise(RuntimeError, "wrong id #{object_id.to_i.to_s}")
              end
            else        
              Backrack::ObjectManager.register(klass.new(), key)
            end
          }  
        end
      
        @classes_hash[klass.name] = klass
        @finders[klass.name] = block
      end
    
      def unregister_class(klass)
        @classes_hash.delete(klass.to_s)
      end
  
      def register(o, key)
        @instances[key] ||= {}
        unless @instances[key][o]
          @instances[key][o] = true
          o.instance_eval { @_session_key = key }
        end
        @cache[key] = Time.now
        return(o)
      end
    
      def instance(key, o_class, object_id=nil)
        @cache[key] = Time.now
        @finders[o_class.to_s].call(object_id, key)   
      end
    
      def sweep
        t = Time.now - (Backrack.config[:session_expiration_time] || 600)
        force_gc = false
        @cache.each_pair{|key, time| 
          if time < t
            @cache.delete(key)
            @instances.delete(key).each{|o|
              o.dispose if o.respond_to?(:dispose)
            }
            force_gc = true
          end
        }
        GC.start if force_gc
      end
    end
  
    init
  end
end