class Backrack::Html
  unless defined?(@@do_it_once)
    Backrack::Dispatcher.register([
        "*/*;q=1",
        "text/xml;q=0.5",
        "application/xml;q=0.5",
        "application/xhtml+xml",
        "text/*;q=0.5",
        "text/html",
        "text/plain;q=0.5"], self)
    @@do_it_once = true
  end
  
  def self.extensions
    ["rhtml", "erb", "haml", "rb"].join(",")
  end

  def self.render_template(path, obinding)
    template_path = Backrack.config[:view]
    search_path = File.join(template_path,"#{path}.{#{extensions}}")

    unless obinding.is_a?(Binding) || obinding.is_a?(Proc)
      obinding = obinding.instance_eval{ binding }
    end

    file_path = Dir.glob(search_path)[0]
    Backrack.logger.debug("searching: "+search_path+" rendering: #{file_path}")
    if file_path
      template = File.read(file_path)
      ext = File.extname(file_path)
    else
      return(nil)
    end

    begin
      result = case ext
      when ".erb", ".rhtml"
        template = ERB.new(File.read(file_path))
        template.filename = file_path
        template.result(obinding)
      when ".haml"
        template = Haml::Engine.new(File.read(file_path))
        template.filename = file_path
        template.render(obinding)
      when ".rb"
        th = Thread.new{
          file = File.read(file_path)
          $SAFE = 1
          eval(file, obinding, file_path)
        }
        th.value
      else
        raise RuntimeError, "unsupported file type #{ext}"
      end
    rescue Exception => e
      Backrack.logger.debug "In file: #{file_path}\n#{e}\n#{e.backtrace.join("\n")}"
      raise
    end
    result
  end
  
  def self.render(object, method_name, binding)
    klass = (object.class==Class) ? object : object.class
    
    ka = klass.ancestors[0..klass.ancestors.index(Object)-1]
    ancestors_path = "{" + ka.map{|k| k.to_s.split("::").map(&:underscore).join("/")}.join(",") +"}"
    if (object.class==Class)
      search_path = ancestors_path + "/{class.#{method_name},#{method_name}}"
    else
      search_path = ancestors_path + "/#{method_name}"
    end
    
    render_template(search_path, binding)
  end
  
  def self.layout(object, binding)
    klass = (object.class==Class) ? object : object.class
    ka = klass.ancestors[0..klass.ancestors.index(Object)-1]
    ancestors_path = "{" + ka.map{|k| k.to_s.split("::").map(&:underscore).join("/")}.join(",") +"}"

    render_template(ancestors_path, binding)
  end

  def self.render_exception(e)
    klass = e.class
    ka = klass.ancestors[0..klass.ancestors.index(Object)-1]
    ancestors_path = "{"+ ka.map{|k| k.to_s.split("::").map(&:underscore).join("/")}.join(",") + "}"
    file_path = "exceptions/#{ancestors_path}"
    proxy = Backrack::TextProxy.new
    pr = proxy._proxy_for(e, self)
    binding = pr._get_binding_with_block { e }

    Backrack.logger.warn "#{e}\n#{e.backtrace.join("\n")}"

    render_template(file_path, binding) || "<pre>#{Rack::Utils.escape_html(e.to_s)}\n#{Rack::Utils.escape_html(e.backtrace.join("\n"))}</pre>"
  end
  
  ######################################################################
  
  attr_accessor :request, :response, :params, :session
  
  def run(object, method_name, lparams, &block)
    locale = request.env["rack.locale"]
    proxy = Backrack::TextProxy.new(locale)
    pr = proxy._proxy_for(object, self)
    
    special_params = {
      :request => @request,
      :response => @response,
      :params => @params,
      :session => @session,
    }

    method = object.method(method_name)
    case lparams
    when Hash
      lparams = method.params.map{|name|
        lparams[name.to_sym] || lparams[name.to_s] || special_params[name]
      }
    when Array
      method.params.each_with_index do |name, i|
        lparams.insert(i, special_params[name]) if special_params.keys.include?(name)
      end
      a = method.arity # lets check arity
      # we make sure that method get all parameters she can handle
      a < 0 ? lparams : a == 0 ? [] : lparams[0..a-1] # ugly but hey it;s 1:30AM!!
    end

    if (@request.xhr? || !@rendered_layout_already) && object.respond_to?(:flash)
      object.flash ||= Hash.new
    end

    Backrack.logger.debug("runing #{method_name} on #{object}")

    catch(:redirect) do
      if @request.xhr? || @rendered_layout_already
        begin
          result = object.send(method_name, *lparams, &block)
          binding = pr._get_binding_with_block { result }
          Backrack::Html.render(object, method_name, binding) || result
        rescue Exception => e
          Backrack::Html.render_exception(e)
        end
      else
        result = begin
          @rendered_layout_already = true
          result = self.run(object, method_name, lparams, &block)
          return(result) if result.kind_of?(Rack::Response)
          binding = pr._get_binding_with_block { result }
          Backrack::Html.layout(object, binding) || result
        rescue Exception => e
          Backrack::Html.render_exception(e)
        end
      end
    end
  end
end
