$: << "lib"

require 'rubygems'

require 'spec'
require 'rack/mock'

require 'json'

require 'lib/backrack'

describe Backrack do
  before(:all) do
    Backrack.initialize!
    Backrack.config[:path] = "test"
    Backrack.discover!
    Backrack.load_serializers(["text", "json_rpc"])
  end
  
  before(:each) do
    @req = Rack::MockRequest.new(Backrack::Dispatcher)  
    @params = {
      "Content-Type" => 'application/json',
      'HTTP_ACCEPT' => 'application/json',
    }
  end
  
  it "should properly execute method with GET" do
    res = @req.get("/FancyName/ping", @params)
    
    res.should be_ok
    ret = nil
    lambda{ ret = JSON.load(res.body) }.should_not raise_error
    ret['result'].should eql('pong')
  end
  
  it "should properly execute method with params with GET" do
    res = @req.get("/FancyName/test_args?a1=1&b1=2&c1=3", @params)    
    
    ret = JSON.load(res.body)
    ret['result'].should == ['1','2','3']
  end

  it "should properly execute method with params with POST" do
    request = {"method" => "echo", "params" => [1, 2, 3], "verion" => "1.0"}.to_json
    params = @params.merge({
        "rack.input" => StringIO.new(request)
      })
    res = @req.post("/FancyName", params)    
    
    ret = JSON.load(res.body)
    ret['result'].should == [1,2,3]
  end

  it "should properly execute method with params using JSON POST" do
    request = {"method" => "test_args", "params" => {"a1" => "1", "b1" => "2", "c1" => "3"}, "verion" => "1.0"}.to_json
    params = @params.merge({
        "rack.input" => StringIO.new(request)
      })
    res = @req.post("/FancyName", params)    
    
    ret = JSON.load(res.body)
    ret['result'].should == ["1","2","3"]
  end
  
  it "should call methods with special arguments correctly" do
    request = {"method" => "test_special_params", "params" => [], "verion" => "1.0"}.to_json
    params = @params.merge({
        "rack.input" => StringIO.new(request)
      })
    res = @req.post("/FancyName", params)    
    
    ret = JSON.load(res.body)
    ret['result'].should == "i'm ok"
  end

  it "should call methods with special arguments correctly" do
    res = @req.post("/FancyName/test_special_params")    
    
    res.body.should == "i'm ok"
  end
  
  after(:all) do
    #Process.kill("TERM",@pid)   
  end
  
end

# EOF
