require 'test/unit'
require 'shoulda'

require 'tests/test_helper'
require 'json'

Backrack.logger.level = Logger::DEBUG

class TestBackrack < Test::Unit::TestCase
  context "Backrack" do
    setup do
      Backrack.load_serializers(["text", "json_rpc", "xml_rpc"])
      @req = Rack::MockRequest.new(Backrack::Dispatcher)  
      @params = {
        "Content-Type" => 'application/json',
        'HTTP_ACCEPT' => 'application/json',
      }
    end
  
    should "properly execute method with GET" do
      res = @req.get("/FancyName/ping", @params)
    
      assert res
      ret = nil
      assert_equal(res.status, 200, "Failure message.")
      assert_nothing_raised(Exception) { ret = JSON.load(res.body) }
      assert_equal ret['result'], 'pong'
    end
  
    should "properly execute method with params with GET" do
      res = @req.get("/FancyName/test_args?a1=1&b1=2&c1=3", @params)    
    
      ret = JSON.load(res.body)
      assert_equal ret['result'],  ['1','2','3']
    end

    should "properly execute method with params with POST" do
      request = {"method" => "echo", "params" => [1, 2, 3], "verion" => "1.0"}.to_json
      params = @params.merge({
          "rack.input" => StringIO.new(request)
        })
      res = @req.post("/FancyName", params)    
    
      ret = JSON.load(res.body)
      assert_equal ret['result'], [1,2,3]
    end

    should "properly execute method with params using JSON POST" do
      request = {"method" => "test_args", "params" => {"a1" => "1", "b1" => "2", "c1" => "3"}, "verion" => "1.0"}.to_json
      params = @params.merge({
          "rack.input" => StringIO.new(request)
        })
      res = @req.post("/FancyName", params)    
    
      ret = JSON.load(res.body)
      assert_equal ret['result'], ["1","2","3"]
    end
    
    should "call methods with only special arguments correctly " do
      res = @req.get("/FancyName/test_special_params")    
    
      assert_equal(res.status, 200)
      assert_equal res.body, "i'm ok"
    end
  
    should "call methods with special arguments correctly" do
      request = {"method" => "test_special_params", "params" => [], "verion" => "1.0"}.to_json
      params = @params.merge({
          "rack.input" => StringIO.new(request)
        })
      res = @req.post("/FancyName", params)    
    
      ret = JSON.load(res.body)
      assert_equal ret['result'], "i'm ok"
    end


  end
end