require 'test/unit'
require 'rubygems'
require 'shoulda'

require 'tests/test_helper'
require 'json'

require 'backrack/client'

class NetHttpImpostor < Rack::MockRequest
  def post(uri, opts, headers=nil)
    res = if headers
      headers["rack.input"] = StringIO.new(opts)
      super(uri, headers)
    else
      super(uri, opts)
    end
    
    [res.status, res.body]
  end
  
  def finish
    
  end
end

class Backrack::TestClient < Backrack::Client
  def initialize
    
  end
  
  def connect
    NetHttpImpostor.new(Backrack::Dispatcher) 
  end
end

class TestClient < Test::Unit::TestCase
  context "Backrack Client" do
    setup do
      Backrack.load_serializers(["json_rpc", "xml_rpc", "text"])
      @client = Backrack::TestClient.new
    end
  
    should "get class FancyName, that is child of Backrack::RPCProxy" do
      ::CFancyName = @client.get_class "FancyName" 
      assert CFancyName.ancestors.include?(Backrack::RPCProxy)
      assert_equal(CFancyName.name, "FancyName")
      assert(CFancyName.respond_to?(:connect_to))
      assert(CFancyName.respond_to?(:local_new))
    end
    
    should "get instance of class FancyName, that is child of Backrack::RPCProxy" do
      assert_nothing_raised(RPCError) { ::CFancyName = @client.get_class "FancyName" } 
      assert CFancyName.ancestors.include?(Backrack::RPCProxy)
      assert_equal(CFancyName.name, "FancyName")
      assert(CFancyName.respond_to?(:connect_to))
      assert(CFancyName.respond_to?(:local_new))
      assert_nothing_raised(RPCError) { @remote = CFancyName.new(1,2)} 
      assert_equal(@remote.class, CFancyName)
    end
    
  
    should "All calls to FancyName should be correct" do
      DFancyName = @client.get_class "FancyName"
      @local = FancyName.new(1,2)
      @remote = DFancyName.new(1,2)
      assert(@remote, "Remote is nil")
      assert(@local, "Local is nil")
      
      [:bar, :ping, :test_array, :foo].each do |f|
        assert_equal FancyName.send(f), DFancyName.send(f)
        assert_equal @remote.send(f), @local.send(f)
      end
      
      assert_equal(DFancyName.test_hash(1,2), {"1"=>2})
      assert_equal(DFancyName.test_args(1, 2, 3), [1,2,3])
      assert_equal(DFancyName.echo("foo"), ["foo"])
      assert_raise(RPCError) { DFancyName.raiser }   
    end   
  end
end
